package fr.faraphel.m1_pe_kafka.kafka

import org.apache.kafka.common.serialization.Serdes
import org.apache.kafka.streams.KafkaStreams
import org.apache.kafka.streams.StreamsBuilder
import org.apache.kafka.streams.StreamsConfig
import org.apache.kafka.streams.Topology
import org.apache.kafka.streams.kstream.KStream
import java.util.*


/**
 * A kafka converter.
 * Take a value from a topic and convert it into another value in a second topic thanks to a conversion function.
 * @param inputTopic the input kafka topic name
 * @param outputTopic the output kafka topic name
 */
class Converter(
    private val inputTopic: String,
    private val outputTopic: String,
) {

    private val properties = Properties().apply {  ///< the kafka properties
        this[StreamsConfig.APPLICATION_ID_CONFIG] = "converter"
        this[StreamsConfig.DEFAULT_KEY_SERDE_CLASS_CONFIG] = Serdes.StringSerde::class.java.name
        this[StreamsConfig.DEFAULT_VALUE_SERDE_CLASS_CONFIG] = Serdes.DoubleSerde::class.java.name
        this[StreamsConfig.BOOTSTRAP_SERVERS_CONFIG] = System.getenv("KAFKA_BOOTSTRAP_SERVERS")
            ?: throw IllegalArgumentException("Missing environment variable: KAFKA_BOOTSTRAP_SERVERS")
    }

    private val topologyBuilder = StreamsBuilder().apply {  ///< the topology builder
        // input
        val inputStream: KStream<String, Double> = this.stream(inputTopic)  ///< the input stream
        // output
        val outputStream: KStream<String, Double> = inputStream  ///< the output stream
            .mapValues { value -> value * 2 }  // convert the values of the pair stream with this function | TODO(Faraphel): use a custom function
        outputStream.to(outputTopic)  // set the output of the stream to this topic
    }
    private val topology: Topology = this.topologyBuilder.build()  ///< the stream topology

    private val streams: KafkaStreams = KafkaStreams(this.topology, this.properties)  ///< the stream

    /**
     * Start the conversion process
     */
    fun start() = streams.start()

    /**
     * Pause the conversion process
     */
    fun pause() = streams.pause()

    /**
     * Resume a paused conversion process
     */
    fun resume() = streams.resume()

    /**
     * Stop a conversion process
     */
    fun stop() = streams.close()
}